/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AttachmentCount.
 *
 * The Initial Developer of the Original Code is
 *   alta88 <alta88@gmail.com>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2013
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

if (Cc == undefined)
  var Cc = Components.classes;
if (Ci == undefined)
  var Ci = Components.interfaces;
if (Cr == undefined)
  var Cr = Components.results;
if (Cu == undefined)
  var Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/NetUtil.jsm");
Cu.import("resource:///modules/gloda/log4moz.js");
Cu.import("resource:///modules/StringBundle.js");

var attachmentCount = {
  get log() {
    var logger = Log4Moz.getConfiguredLogger("extensions.AttachmentCount");
    this.__defineGetter__("log", function() { return logger });
    return this.log;
  },

  get strings() {
    return this._strings ? this._strings :
      this._strings = new StringBundle("chrome://attachmentcount/locale/ac.properties");
  },

  get alwaysShowCount() {
    return Services.prefs.getBoolPref("extensions.attachmentcount.alwaysShowCount");
  },

  get showAllPartsPref() {
    return Services.prefs.getIntPref("mailnews.display.html_as") == 4;
  },

  get downloadPrefs() {
    return JSON.parse(
      Services.prefs.getCharPref("extensions.attachmentcount.downloadPrefs"));
  },
  get imapAllowDownload()  {return this.downloadPrefs.imapAllowDownload;},
  get saneBodySize()       {return this.downloadPrefs.saneBodySize;},
  get imapPartsOnDemand()  {return this.downloadPrefs.imapPartsOnDemand;},
  // Throws if false for nntp.
  get encryptedParts()     {return true;},

  get threadTreeChildren() {
    return GetThreadTree().getElementsByTagName("treechildren")[0];
  },
  get previewContentType() {
    return Services.prefs.getCharPref("extensions.attachmentcount.previewContentTypes").
                          indexOf(this.lastContentType) != -1;
  },
  get previewTitle()       {
    return this.strings.get("attachmentPreviewTitle", [this.lastName]);
  },

  /**
   * nsITreeview.
   */
  isEditable:          function(row, col) {return false;},
  getRowProperties:    function(row)      {return "";},
  isString:            function()         {return true;},
  getSortLongForRow:   function(msgHdr)   {return 0;},
  getSortStringForRow: function(msgHdr)   {
    let countInfo = this.getCount(msgHdr);
    if (!countInfo)
      return "";
    let ext = "";
    if (countInfo[this.CountInfo.kAttachments] == 1 &&
        countInfo[this.CountInfo.kName]) {
      // Get the file ext.  We want to group same type 1 count items
      // together.
      let name = countInfo.slice(4, countInfo.length).join();
      let split = name.split(".");
      ext = split[split.length - 1].toLowerCase();
    }
    let countStr = countInfo[this.CountInfo.kAttachments].toString();
    while (countStr.length < 3)
      countStr = "0" + countStr;
    let sortStr = countInfo[this.CountInfo.kFlag] + countStr + ext;
this.log.debug("getSortLongForRow: sortStr:ext:countInfo - "+sortStr+":"+ext+":"+countInfo);
    return sortStr;
  },
  getImageSrc:         function(row, col) {
    if (this.showAllPartsPref || this.alwaysShowCount)
      return null;
    let msgHdr = gDBView.getMsgHdrAt(row);
    if (!msgHdr)
      return null;
    let countInfo = this.getCount(msgHdr);
    if (!countInfo || countInfo[this.CountInfo.kAttachments] != 1 ||
        [this.kDeleted, this.kDetached].
         indexOf(countInfo[this.CountInfo.kFlag]) != -1)
      return null;

    let iconStr = "moz-icon://sheep?size=16";
    let contentType = "";
    if (countInfo[this.CountInfo.kName]) {
      if (countInfo[this.CountInfo.kContentType])
        contentType = "&contentType=" + countInfo[this.CountInfo.kContentType];
      iconStr = "moz-icon://" + countInfo[this.CountInfo.kName] +
                "?size=16" + contentType;
    }
this.log.trace("getImageSrc: icon - "+iconStr);
    return iconStr;
  },
  cycleCell:           function(row, col) {},
  getCellProperties:   function(row, col) {
//this.log.debug("getCellProperties: col - "+col.id);
    let props = [];
    props.push("attachmentCountCol");
    if (this.alwaysShowCount)
      return props.join(" ");
    let msgHdr = gDBView.getMsgHdrAt(row);
    if (!msgHdr)
      return props.join(" ");
    let countInfo = this.getCount(msgHdr);
    if (!countInfo)
      return props.join(" ");
this.log.trace("getCellProperties: count - "+countInfo);
    if (countInfo[this.CountInfo.kFlag] == this.kDeleted)
      props.push("allDeleted");
    if (countInfo[this.CountInfo.kFlag] == this.kDetached)
      props.push("allDetached");
    if (countInfo[this.CountInfo.kFlag] == this.kRemote)
      props.push("allRemote");
    if (countInfo[this.CountInfo.kFlag] == this.kUnknown)
      props.push("allUnknown");
    if (countInfo[this.CountInfo.kAttachments] == 1 &&
        [this.kDeleted, this.kUnknown, this.kRemote].
         indexOf(countInfo[this.CountInfo.kFlag]) == -1)
      props.push("previewSingle");
    if (row == attachmentCount.previewRow)
      props.push("previewing");
    return props.join(" ");
  },
  getCellText:         function(row, col) {
    let msgHdr = gDBView.getMsgHdrAt(row);
    if (!msgHdr)
      return null;
    let countInfo = this.getCount(msgHdr);
    if (countInfo && countInfo[this.CountInfo.kFlag] == this.kUnknown)
      return "?";
    if (!countInfo || !countInfo[this.CountInfo.kAllBodyParts])
      return null;
    return this.showAllPartsPref ? countInfo[this.CountInfo.kAllBodyParts] :
                                   countInfo[0] == 27 ? 555 : countInfo[0] == 7 ? 55 :
                                   countInfo[this.CountInfo.kAttachments];
  },

  kDeleted:        0,
  kDetached:       1,
  kRemote:         2,
  kNormal:         3,
  kUnknown:        4,
  CountInfo: {
    kAttachments:  0,
    kAllBodyParts: 1,
    kFlag:         2,
    kContentType:  3,
    kName:         4
  },
  get CountInfoLength() {
    return Object.keys(this.CountInfo).length;
  },

  // A shift-click on the attachment count column header will enter reset mode,
  // any message that has been scrolled into view will be reset on a second
  // shift-click that exits reset mode.  The attachmentCounts property will
  // be derived again from the mime message.
  reset:               false,
  toggleReset:         function(aEvent) {
    attachmentCount.reset = !attachmentCount.reset;
    GetThreadTree().treeBoxObject.invalidate();
  },

  gettingCount:        false,
  getCountProperty:    function(aMsgHdr)  {
    let counts = aMsgHdr.getProperty("attachmentCounts");
this.log.trace("getCountProperty: prop:subject - "+counts+":"+aMsgHdr.mime2DecodedSubject);
    if (counts.length == 0)
      return null;
    counts = counts.split(",");
    counts[0] = parseInt(counts[0]);
    counts[1] = parseInt(counts[1]);
    counts[2] = parseInt(counts[2]);
    return counts;
  },
  setCountProperty:    function(aMsgHdr, aVal)  {
    aMsgHdr.setProperty("attachmentCounts", aVal);
  },
  getCount:            function(aMsgHdr)  {
//this.log.debug("getCount: START reset:subject - "+this.reset+":"+aMsgHdr.mime2DecodedSubject);
    let msgHdr = aMsgHdr;
    this.lastMsgHdr = aMsgHdr;

    if (!(msgHdr.flags & Ci.nsMsgMessageFlags.Attachment))
      // No attachments.
      return null;

    if (this.reset) {
      this.setCountProperty(msgHdr, null);
      return null;
    }

    let countInfo = this.getCountProperty(msgHdr);
    if (countInfo)
      // Return the already stored attachmentCount property.
      return this.lastCountInfo = countInfo;

//this.log.debug("getCount: count:subject - "+countInfo+":"+msgHdr.mime2DecodedSubject);
    // TODO: check if logged in.
    let imapLoggedIn = true;

    if (msgHdr.folder.server.type == "imap" &&
        (!this.imapAllowDownload || !imapLoggedIn)) {
      // If there is no count already then not allowing download throws
      // in MsgHdrToMimeMessage, and if not signed in can't get a count.
      countInfo = new Array(attachmentCount.CountInfoLength),
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kUnknown;
      return countInfo;
    }

    if (this.gettingCount)
      return null;
    this.gettingCount = true;

    // Get the count from message property attachmentCount.  If not there,
    // derive from the mime message and set it.
    // XXX: MsgHdrToMimeMessage used to be crap.  Thanks to protz for
    // fixing the api.  However, it still chokes on nntp.
this.log.trace("getCount: prefs: "+this.downloadPrefs.toSource());
    MsgHdrToMimeMessage(msgHdr, null, this.mimeMsgCallback,
                        this.imapAllowDownload,
                        { saneBodySize:          this.saneBodySize,
                          partsOnDemand:         this.imapPartsOnDemand,
                          examineEncryptedParts: this.encryptedParts });

    // Wait for the async call to finish; return null values.
    return null;
  },

  MsgSortByAttachmentCount: function() {
    if (!gDBView || !gDBView.db)
      return;

    gDBView.db.dBFolderInfo.setProperty("customSortCol", "attachmentCountCol")
    MsgSortThreadPane("byCustom");
  },

  // Preview single attachment.
  preview:             function(aMsgHdr)  {
this.log.debug("preview: START lastMsgHdr.subject - "+aMsgHdr.mime2DecodedSubject);
this.log.debug("preview: CONTINUE count - "+this.lastCountInfo);
this.log.debug("preview: CONTINUE lastName - "+this.lastName);
this.log.debug("preview: CONTINUE lastContentType - "+this.lastContentType);
this.log.debug("preview: CONTINUE lastUrl - "+this.lastUrl);

    if (!this.previewContentType || !this.lastUrl) {
      this.doPreview = false;
      return;
    }

    let url = Services.io.newURI(this.lastUrl, null, null);
    let channel = Services.io.newChannelFromURI(url);
    channel.asyncOpen(this.StreamListener, null);
  },

  // Observer for custom column and show allBodyParts pref.
  Observer:            {
    observe: function(aSubject, aTopic, aData) {
      if (aTopic == "MsgCreateDBView" && gDBView && ("attachmentCount" in window)) {
attachmentCount.log.debug("Observer: addColumnHandler - attachmentCountCol");
        gDBView.addColumnHandler("attachmentCountCol", attachmentCount);
        attachmentCount.reset = false;
      }
      else if (aTopic == "nsPref:changed" && aData == "mailnews.display.html_as")
        GetThreadTree().treeBoxObject.invalidate();
    }
  },

  // Listener for header re-add on attachment detach/delete.
  Listener:            {
    QueryInterface:       XPCOMUtils.generateQI([
                            Ci.nsISupports,
                            Ci.nsIDBChangeListener
                          ]),
    onHdrFlagsChanged:    function(aHdrChanged, aOldFlags,
                                   aNewFlags, aInstigator) {},
    onHdrDeleted:         function(aHdrChanged, aParentKey,
                                   aFlags, aInstigator)    {},
    onParentChanged:      function(aKeyChanged, oldParent,
                                   newParent, aInstigator) {},
    onAnnouncerGoingAway: function(aInstigator)            {},
    onReadChanged:        function(aInstigator)            {},
    onJunkScoreChanged:   function(aInstigator)            {},
    onHdrPropertyChanged: function(aHdrToChange, aPreChange,
                                   aStatus, aInstigator)   {},
    onEvent:              function(aDB, aEvent)            {},

    onHdrAdded:           function(aHdrChanged, aParentKey,
                                   aFlags, aInstigator)    {
      // On attachment detach/delete, clear property to recreate new state.
attachmentCount.log.debug("onHdrAdded: START ");
      gFolderDisplay.view.dbView.db.RemoveListener(attachmentCount.Listener);
      aHdrChanged.setProperty("attachmentCounts", null);
attachmentCount.log.debug("onHdrAdded: attachmentCount reset - "+aHdrChanged.subject);
    }
  },

  // Listener for streaming attachment.
  StreamListener:      {
    QueryInterface:       XPCOMUtils.generateQI([
                            Ci.nsISupports,
                            Ci.nsIStreamListener,
                            Ci.nsIRequestObserver
                          ]),
    onStartRequest:       function (aRequest, aContext) {
      this.unicodeConverter =
           Cc["@mozilla.org/intl/scriptableunicodeconverter"].
           createInstance(Ci.nsIScriptableUnicodeConverter);
      this.unicodeConverter.charset = "UTF-8";
      this.data = "";
      this.maxLimit = 4000;
    },
    onStopRequest:        function (aRequest, aContext, aStatusCode) {
attachmentCount.log.debug("StreamListener.onStopRequest: final data len - "+
                          this.data.length);
attachmentCount.log.debug("StreamListener.onStopRequest: final data - "+
                          this.data);
      let title = attachmentCount.previewTitle + "\n\n";
      let preview = title + this.data + "\n";
      attachmentCount.threadTreeChildren.setAttribute("tooltiptext", preview);
    },
    onDataAvailable:      function (aRequest, aContext, aStream, aOffset, aCount) {
attachmentCount.log.debug("StreamListener.onDataAvailable: aCount - "+aCount);

      let bytes = NetUtil.readInputStreamToString(aStream, aCount);

      if (this.data.length >= this.maxLimit)
        // Don't construct a data buffer bigger than the maxLimit.  Is
        // there a way to not read the whole stream..
        return;
      let byteCount = this.maxLimit - this.data.length;
      byteCount = byteCount > aCount ? aCount : byteCount;

attachmentCount.log.debug("StreamListener.onDataAvailable: data length - "+
                          this.data.length);

      // Each character of the string is a byte of a UTF-8 string. Use
      // charCodeAt.
      let array = [];
      for (let i = 0; i < byteCount; ++i)
        array[i] = bytes.charCodeAt(i);
attachmentCount.log.debug("StreamListener.onDataAvailable: array length - "+
                          array.length);

      this.data += this.unicodeConverter.convertFromByteArray(array, array.length);

attachmentCount.log.debug("StreamListener.onDataAvailable: +=data length - "+
                          this.data.length);
    }
  },

  mimeMsgCallback:     function(aMsgHdr, aMimeMsg) {
    let part, val, partname, partcontentType,
        count = {c:0, m:0, b:0, a:0, u:0},
        countInfo = new Array(attachmentCount.CountInfoLength),
        deletedCount = 0,
        detachedCount = 0,
        remoteCount = 0,
        msgHdr = aMsgHdr;
attachmentCount.log.debug("MsgHdrToMimeMessage: callback, aMsgHdr - "+aMsgHdr.toSource());

    if (aMimeMsg == null) {
attachmentCount.log.debug("MsgHdrToMimeMessage: aMimeMsg is null ");
      // Couldn't get a mime message.  Always true for nntp.
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kUnknown;
      attachmentCount.setCountProperty(msgHdr, countInfo);
      attachmentCount.gettingCount = false;
      GetThreadTree().treeBoxObject.invalidate();
      return;
    }

attachmentCount.log.debug("MsgHdrToMimeMessage: aMimeMsg: "+aMimeMsg.prettyString());
attachmentCount.log.trace("MsgHdrToMimeMessage: aMimeMsg: "+aMimeMsg.toSource());

    // Implement recursive parser ourselves..
    function counter(parts) {
      for (let [index, part] in Iterator(parts)) {

if (!attachmentCount.reset)
attachmentCount.log.debug("counter: part - "+part.toSource());

        if (part instanceof MimeContainer) {
          count.c++;
if (!attachmentCount.reset)
attachmentCount.log.debug("counter: MimeContainer partName:count:"+
                          "contentType:name - "+
                          part.partName+" : "+count.c+" : "+
                          part.contentType+" : "+part.name);
        }

      if (part instanceof MimeMessage) {
        count.m++;
        partname = part.name ? part.name.match(/\./g)
                             ? part.name : part.name + ".1" : ".1";
        partcontentType = part.contentType;
        attachmentCount.lastName = part.name;
        attachmentCount.lastContentType = part.contentType;
        attachmentCount.lastUrl = part.url;
if (!attachmentCount.reset)
attachmentCount.log.debug("counter: MimeMessage partName:count:"+
                    "contentType:name - "+
                    part.partName+" : "+count.m+" : "+
                    part.contentType+" : "+partname);
        }

        if (part instanceof MimeBody) {
          count.b++;
if (!attachmentCount.reset)
attachmentCount.log.debug("counter: MimeBody partName:count:"+
                    "contentType:name - "+
                    part.partName+" : "+count.b+" : "+
                    part.contentType+" : "+part.name);
        }

        if (part instanceof MimeMessageAttachment) {
          count.a++;
          // Use a ".1" extension if none or no name, to show a blank icon.
          partname = part.name ? part.name.match(/\./g)
                               ? part.name : part.name + ".1" : ".1";

          if (part.contentType == "multipart/appledouble")
            // Handle appledouble part types, contains 2 children, the
            // 2nd of which has the real content type.
            partcontentType = part.parts[1].contentType;
          else
            partcontentType = part.contentType;

          attachmentCount.lastName = part.name;
          attachmentCount.lastContentType = partcontentType;
          attachmentCount.lastUrl = part.url;
if (!attachmentCount.reset)
attachmentCount.log.debug("counter: MimeMessageAttachment partName:count:"+
                          "contentType:name - "+
                          part.partName+" : "+count.a+" : "+
                          partcontentType+" : "+partname);

          // Deleted attachments.
          if (partcontentType == "text/x-moz-deleted")
            deletedCount++;

          // Detached attachments and Feed enclosures (isExternal=true)
          // are correctly returned as MimeMessageAttachment in Tb8.0+.
          // Differentiate between them.
          if (part.isExternal) {
attachmentCount.log.debug("counter: MimeMessageAttachment part.url:"+part.url);
            if (part.url.match(/^(http|ftp)/))
              // A feed message url is 'remote', http or ftp.  A feed
              // messge doesn't have to be in a feed folder.
              remoteCount++;
            else
              detachedCount++;
          }
        }

        if (part instanceof MimeUnknown) {
          count.u++;
if (!attachmentCount.reset) {
attachmentCount.log.debug("counter: MimeUnknown part headers.content-type: "+
                          part.headers["content-type"][0]);
attachmentCount.log.debug("counter: MimeUnknown partName:count:"+
                          "contentType:name - "+
                          part.partName+" : "+count.u+" : "+
                          part.contentType+" : "+part.name);
}
        }

        if (part.parts && part.contentType != "multipart/appledouble")
          counter(part.parts);

      } // for
    } // counter()

    counter(aMimeMsg.parts);
    let attachments = count.m + count.a;
    let allBodyParts = attachments + count.b + count.u;
    countInfo[attachmentCount.CountInfo.kAttachments] = attachments;
    countInfo[attachmentCount.CountInfo.kAllBodyParts] = allBodyParts;
    if (attachments == 0) {
      // The msgHdr says we have an attachment but we didn't count any.
      // There must be an error in parsing, or there was no 'name' in
      // content-type test; set unknown indicator and log a warning.
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kUnknown;
attachmentCount.log.warn("counter: MimeUnknown couldn't get a count though " +
                         "attachment(s) exist");
    }
    // All of a certain type get their own decoration..
    else if (attachments == deletedCount)
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kDeleted;
    else if (attachments == detachedCount)
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kDetached;
    else if (attachments == remoteCount)
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kRemote;
    else
      countInfo[attachmentCount.CountInfo.kFlag] = attachmentCount.kNormal;

    if (attachments == 1) {
      countInfo[attachmentCount.CountInfo.kContentType] = partcontentType;
      countInfo[attachmentCount.CountInfo.kName] = partname;
    }

    attachmentCount.setCountProperty(msgHdr, countInfo);
    attachmentCount.gettingCount = false;
    GetThreadTree().treeBoxObject.invalidate();

    if (attachmentCount.doPreview)
      attachmentCount.preview(msgHdr);

if (!attachmentCount.reset)
attachmentCount.log.debug("MsgHdrToMimeMessage: Finish: countInfo:Subject - "+
                          countInfo+":"+msgHdr.subject);

  }
}; // attachmentCount


AttachmentInfo.prototype.detach = function AttachmentInfo_detach(aSaveFirst) {
attachmentCount.log.trace("AttachmentInfo.detach: START "+aSaveFirst);
  // The sequence of code flow is async for detach, but not delete, with
  // no way to wait for the final confirm dialog, therefore we use this
  // listener.  Note that on a cancel (no way to wait for that either) the
  // listener will remain until a folder is changed.
  messenger.detachAttachment(this.contentType, this.url,
                             encodeURIComponent(this.name),
                             this.uri, aSaveFirst);
  gFolderDisplay.view.dbView.db.AddListener(attachmentCount.Listener);
attachmentCount.log.debug("AttachmentInfo.detach: DONE");
};

attachmentCount._HandleMultipleAttachments = HandleMultipleAttachments;
HandleMultipleAttachments = function(attachments, action) {
attachmentCount.log.trace("HandleMultipleAttachments: START "+action);
  attachmentCount._HandleMultipleAttachments(attachments, action);
  gFolderDisplay.view.dbView.db.AddListener(attachmentCount.Listener);
attachmentCount.log.debug("HandleMultipleAttachments: DONE");
};

attachmentCount._TreeOnMouseDown = TreeOnMouseDown;
TreeOnMouseDown = function(aEvent) {
attachmentCount.log.trace("TreeOnMouseDown: START ");

  if (attachmentCount.doPreview) {
    aEvent.stopPropagation();
    attachmentCount.threadTreeChildren.removeAttribute("tooltiptext");
    attachmentCount.doPreview = false;
    let row = attachmentCount.previewRow;
    attachmentCount.previewRow = null;
    GetThreadTree().treeBoxObject.invalidateRow(row);
    return true;
  }

  let row = { }, col = { }, element = { };
  GetThreadTree().treeBoxObject.getCellAt(aEvent.clientX,
                                          aEvent.clientY,
                                          row, col, element);
  row = row.value;
  col = col.value ? col.value.id : null;
  element = element.value;

  if (row == -1 || aEvent.button != 0 ||
      aEvent.target.localName == "treecol" ||
      col != "attachmentCountCol")
    return attachmentCount._TreeOnMouseDown(aEvent);

attachmentCount.log.trace("TreeOnMouseDown: CONTINUE ");
  let countInfo = attachmentCount.lastCountInfo;
  if (countInfo[attachmentCount.CountInfo.kAttachments] != 1 ||
      [attachmentCount.kDeleted, attachmentCount.kUnknown].
       indexOf(countInfo[attachmentCount.CountInfo.kFlag]) != -1)
    return attachmentCount._TreeOnMouseDown(aEvent);

  aEvent.stopPropagation();
  attachmentCount.lastName = null;
  attachmentCount.lastContentType = null;
  attachmentCount.lastUrl = null;
  attachmentCount.previewRow = row.value;
  attachmentCount.lastMsgHdr.setProperty("attachmentCounts", null);
  attachmentCount.doPreview = true;
  return true;
};

attachmentCount._ThreadPaneOnClick = ThreadPaneOnClick;
ThreadPaneOnClick = function(aEvent) {
attachmentCount.log.trace("ThreadPaneOnClick: START ");
  if (aEvent.shiftKey && aEvent.target.id == "attachmentCountCol")
    return attachmentCount.toggleReset(aEvent);

  return attachmentCount._ThreadPaneOnClick(aEvent);
}

Services.obs.addObserver(attachmentCount.Observer, "MsgCreateDBView", false);
Services.prefs.addObserver("mailnews.display.html_as", attachmentCount.Observer, false);
